/*++

	This program is free software; you can redistribute it
	and/or modify it under the terms of the Shared Code
	License (SCL) as published by www.IntellectualHeaven.com;
	either version 1.0, or (at your option) any later version.
	You should have received a copy of the SCL 1.0 license,
	look for file license.txt, along with this software; if not,
	then please write to x_pankaj_x@intellectualheaven.com
	for an electronic copy of the same.

Module Name:

    YArchDecodeView.cpp

Module Description:

	Main view class for YahDecode. It also implements the
	other functionality like decoding a yahoo archived
	message etc.

Author:

    Pankaj Garg		2004-09-17
    
Revision History:

--*/

//System-Includes!!!
#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <commctrl.h>
#include <commdlg.h>
#include <time.h>
#include <richedit.h>
#include <shlwapi.h>


// Local-Includes!!!
#include "YArchDecodeView.h"
#include "YArchDecode.h"
#include "resource.h"


#pragma warning (disable:4244)	// Conversion with possible loss of data
#pragma warning (disable:4267)	// Conversion with possible loss of data



//
// Global registry key for storing information about
// this application
//
#define APP_INFO_REG_KEY		_T("Software\\InellectualHeaven\\YahDecode")


//
// Yahoo user name length limit
//
#define	MAX_YAHOO_USER_NAME_LEN		32


//
// Global registry key where yahoo stores information
//
#define YAHOO_INFO_REG_KEY		_T("Software\\Yahoo\\Pager\\profiles")


//
// Global application instance
//
HINSTANCE ghInstance;


//
// Global function to store edit control data
// This data normally points to the decoded conversation
//
tstring gMsgText;
int gMsgTextPos;


//
// Dialog Proc for About Dialog
//
INT_PTR
CALLBACK
AboutDlgProc(	
		HWND hDlg,
		UINT msg,
		WPARAM wParam,
		LPARAM lParam);


//
// Dialog proc for Yahoo user information for single
// archive file decoding
//
INT_PTR
CALLBACK
UserInfoDlgProc(
		HWND hDlg,
		UINT msg,
		WPARAM wParam,
		LPARAM lParam);


//
// Function to move the dialog to center of the parent
//
void
CenterDialog(HWND hwndDlg);


//
// Function for adding text to the rich edit control
//
DWORD
CALLBACK
EditStreamCallback(
	DWORD_PTR dwCookie,
	LPBYTE pbBuff,
	LONG cb,
	LONG *pcb);


//
// Message List Control related data
//
#define LISTCTRL_DATE_POS				0
#define LISTCTRL_FRIEND_OR_HOST_POS		1
#define LISTCTRL_YOUR_PROFILE_POS		2

ListControlHeaderData gMsgListControlData[] = 
	{
		{ 100,	_T("Date")		},
		{ 150,	_T("Friend or Conference host")	},
		{ 150,	_T("Your Profile")				}
	};


//
// User List Control related data
//
#define USER_LISTCTRL_NAME_POS			0
#define USER_LISTCTRL_ARCHIVING_POS		1

ListControlHeaderData gUserListControlData[] = 
	{
		{ 108,	_T("Active Yahoo IDs")	},
		{ 57,	_T("Archiving")			}
	};


/*++

Routine Name:
	
	CYArchDecode

Routine Description:
	
	Constructor for CYArchDecode

--*/
CYArchDecode::CYArchDecode(HINSTANCE hInstance)
{
    m_hInstance			= hInstance;
	m_MessageBoxFont	= NULL;
	ghInstance			= hInstance;
	m_ShowTimestamp		= false;
	m_RichEdit			= false;
	m_OldComctl32		= true;
    
	// Common controls must be initialized
    InitCommonControls();
}



/*++

Routine Name:
	
	~CYArchDecode

Routine Description:
	
	Destructor for CYArchDecode

--*/
CYArchDecode::~CYArchDecode()
{
	if (m_MessageBoxFont)
	{
		DeleteObject(m_MessageBoxFont);
	}
}


/*++

Routine Name:
	
	GetMainWindow

Routine Description:
	
	Gets handle to the main window

Returns:

	Handle to Main Application Window

--*/
HWND
CYArchDecode::GetMainWindow()
{
	return m_hwnd;
}


/*++

Routine Name:
	
	GetAccel

Routine Description:
	
	Gets handle to loaded accelerator

Returns:

	Handle to Loaded Accelerator

--*/
HACCEL
CYArchDecode::GetAccel()
{
	return m_hAccel;
}



/*++

Routine Name:
	
	InitInstance

Routine Description:
	
	Called when the application starts.	Its job is to create
	and	initialize the main window for the application and
	other controls.

Returns:

	None

--*/

typedef HRESULT (CALLBACK *PFN_DLL_GET_VERSION)(DLLVERSIONINFO *);

void
CYArchDecode::InitInstance()
{
	HMODULE hComctl32 = LoadLibrary(_T("comctl32.dll"));

	if (hComctl32)
	{
		PFN_DLL_GET_VERSION pfnDllGetVersion = 
								(PFN_DLL_GET_VERSION)GetProcAddress(
																hComctl32,
																"DllGetVersion");

		if (pfnDllGetVersion)
		{
			DLLVERSIONINFO dllVerInfo;
			dllVerInfo.cbSize = sizeof(dllVerInfo);

			if (pfnDllGetVersion(&dllVerInfo) == NO_ERROR)
			{
				if (MAKELONG(dllVerInfo.dwMajorVersion, dllVerInfo.dwMinorVersion) >= MAKELONG(5,80))
				{
					m_OldComctl32 = false;
				}
			}
		}

		FreeLibrary(hComctl32);
	}

    CreateMainWindow();
    CreateStatusBar();
	CreateToolBar();
	
	CreateMessageListControl();

	CreateMessageTextBox();
	
	CreateUserTreeControl();
	AttachImageListToTree();
	PopulateUserTree();

	CreateUserListControl();
	PopulateUserListControl();

    ShowWindow(m_hwnd, SW_SHOW);
    UpdateWindow(m_hwnd);
}



/*++

Routine Name:
	
	ExitInstance

Routine Description:
	
	Called just before the application is about to exit
	to perform any house-keeping tasks.

Returns:

	None

--*/
void
CYArchDecode::ExitInstance()
{
	// Save the window size information and post quit
    SaveAppInformation();
    PostQuitMessage(0);
}



/*++

Routine Name:
	
	Refresh

Routine Description:
	
	Refresh all the windows

Returns:

	None

--*/
void
CYArchDecode::Refresh()
{
	PopulateUserTree();
	PopulateUserListControl();
}


/*++

Routine Name:
	
	CreateMainWindow

Routine Description:
	
	Creates Main Application Window

Returns:

	None

--*/
void
CYArchDecode::CreateMainWindow()
{
    static TCHAR szAppName[ ] = _T("YArchDecode");
    WNDCLASSEX wndclass;

    //
	// setup a window class for our use
    //
    wndclass.cbSize				= sizeof(wndclass);
    wndclass.style				= CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc		= ::WndProc;
    wndclass.cbClsExtra			= 0;
    wndclass.cbWndExtra			= 0;
    wndclass.hInstance			= m_hInstance;
	wndclass.hIconSm			=
    wndclass.hIcon				= LoadIcon(m_hInstance, (LPCTSTR)IDI_APP_ICON);
    wndclass.hCursor			= LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground		= (HBRUSH)GetStockObject(WHITE_BRUSH);
    wndclass.lpszMenuName		= (LPTSTR)IDR_MENU;
    wndclass.lpszClassName		= szAppName;

    RegisterClassEx(&wndclass);


    //
    // For windows initialize size and position
    //
    RECT rect;

	//
	// Default values for window position
	//
    rect.left	= 100;
    rect.right	= 800;
    rect.top	= 20;
    rect.bottom = 570;

	//
	// Load any previously saved information for this user
	//
    LoadAppInformation(&rect);


    //
    // create main window
    //
    m_hwnd = CreateWindowEx(0,
                            szAppName,
                            APPLICATION_TITLE,
                            WS_OVERLAPPEDWINDOW,
                            rect.left,
                            rect.top,
                            rect.right - rect.left,
                            rect.bottom - rect.top,
                            NULL,
                            NULL,
                            m_hInstance,
                            NULL);

	if (!m_hwnd)
	{
		return;
	}

	m_hAccel = LoadAccelerators(
							m_hInstance,
							(LPCTSTR)IDR_MENU);
}



/*++

Routine Name:
	
	WndProc

Routine Description:
	
	Our Windows message handling routine

Returns:

	If a message is handled the return value is zero. Otherwise
	its non-zero.
	

--*/
LRESULT
CYArchDecode::WndProc(	
				HWND	inHWnd,
				UINT	nMsg,
				WPARAM	wParam,
				LPARAM	lParam)
{
    switch(nMsg)
    {
		case WM_NOTIFY:
        {
            LPNMHDR		pNM		= (LPNMHDR)lParam;

			//
            // handle custom draw notifications
            //
            if(pNM->hwndFrom == m_hwndMessageList)
            {
				int nSelectedItem = 0;

				switch(pNM->code)
				{
					case NM_CUSTOMDRAW:
					{
						//
						// Custom draw from list view
						//
						LPNMLVCUSTOMDRAW pCD = (LPNMLVCUSTOMDRAW)lParam;

						if(pCD->nmcd.dwDrawStage == CDDS_PREPAINT)
						{
							return CDRF_NOTIFYITEMDRAW;
						}
						else if(pCD->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
						{
							if (pNM->hwndFrom == m_hwndMessageList)
							{
								//
								// Color of list control text can be changed
								// here
								//
								pCD->clrText = RGB(0, 160, 0);

								return CDRF_DODEFAULT;
							}
						}

						break;
					}
					case LVN_KEYDOWN:
					{
						LPNMLVKEYDOWN pLVKeyDown = (LPNMLVKEYDOWN)lParam;

						if (pLVKeyDown->wVKey != VK_DOWN &&
							pLVKeyDown->wVKey != VK_UP)
						{
							break;
						}

						nSelectedItem = ListView_GetSelectionMark(m_hwndMessageList);

						if (pLVKeyDown->wVKey == VK_UP &&
							nSelectedItem != 0)
						{
							nSelectedItem = -1;
						}
						else if(pLVKeyDown->wVKey == VK_DOWN &&
								nSelectedItem < (ListView_GetItemCount(m_hwndMessageList) - 1))
						{
							int nCount = ListView_GetItemCount(m_hwndMessageList);
							nSelectedItem = 1;
						}
						else
						{
							nSelectedItem = 0;
						}
					}
					// NOTICE: Fall through
					case NM_CLICK:
					{
						nSelectedItem += ListView_GetSelectionMark(m_hwndMessageList);

						//XXX-XXX
						if (nSelectedItem >= 0)
						{
							LVITEM lvItem = {0};
							lvItem.mask		= LVIF_PARAM;
							lvItem.iItem	= nSelectedItem;
							
							if (ListView_GetItem(m_hwndMessageList, &lvItem))
							{
								int itemIndex = (int)lvItem.lParam;
								if (itemIndex > 0)
								{	
									ARCHIVE_FILE_DATA &archiveFileData = m_MessageListNodeParams[itemIndex - 1];
									PopulateTextBoxData(archiveFileData);
								}
							}
							else
							{
								nSelectedItem = GetLastError();
							}
						}

						break;
					}
					case NM_DBLCLK:
					{
						//
						// You can process double click here. for now
						// we just add same data over n over
						//
						break;
					}
				}
			}
			else if(pNM->hwndFrom == m_hwndUserTree)
            {
				switch(pNM->code)
				{
					case TVN_SELCHANGED:
					{
						LPNMTREEVIEW pnmtv = (LPNMTREEVIEW)lParam;
						int itemIndex = (int)pnmtv->itemNew.lParam;
						if (itemIndex > 0)
						{	
							//
							// Clear both the list box and text box and related
							// data
							//
							ListView_DeleteAllItems(m_hwndMessageList);
							m_MessageListNodeParams.clear();
							SetMessageTextBoxText(_T(""));

							ARCHIVE_TREE_DATA &archiveTreeData = m_TreeNodeParams[itemIndex - 1];

							if (archiveTreeData.nodeType == NODE_CATEGORY)
							{
								PopulateMessageListCtrlWholeCategory(archiveTreeData);
							}
							else
							{
								PopulateMessageListCtrl(archiveTreeData);
							}
						}

						break;
					}
				}
			}
			else if(pNM->hwndFrom == m_hwndUserList)
			{
				switch(pNM->code)
				{
					case LVN_ITEMCHANGED:
					{
						LPNMLISTVIEW listView = (LPNMLISTVIEW)lParam;

						if (listView->uChanged & LVIF_STATE)
						{
							if ((listView->uOldState == 4096 && listView->uNewState == 8192) ||
								(listView->uOldState == 8192 && listView->uNewState == 4096))
							{
								TCHAR userName[MAX_YAHOO_USER_NAME_LEN + 1] = {0};
								ListView_GetItemText(m_hwndUserList, listView->iItem, 0, userName, MAX_YAHOO_USER_NAME_LEN);

								if (ListView_GetCheckState(m_hwndUserList, listView->iItem))
								{
									EnableArchiveForYahooUser(userName, listView->iItem, TRUE);
								}
								else
								{
									EnableArchiveForYahooUser(userName, listView->iItem, FALSE);
								}
							}
						}

						break;
					}
				}
			}
			break;
        }
		case WM_CTLCOLORSTATIC:
		{
			HDC hdc = (HDC)wParam;
			HWND hwndCtl = (HWND)lParam;

			if (hwndCtl == m_hwndMessageTextBox)
			{
				if (m_RichEdit)
				{
					SetBkColor(hdc, RGB(218, 214, 192));
				}
				else
				{
					if (!m_MessageBoxFont)
					{
						m_MessageBoxFont = CreateFont(
												14, 0, 0, 0, FW_THIN,
												FALSE,FALSE,FALSE,
												ANSI_CHARSET,
												OUT_DEFAULT_PRECIS,
												CLIP_DEFAULT_PRECIS,
												DEFAULT_QUALITY,
												DEFAULT_PITCH | FF_SWISS,
												_T("Courier New"));
					}

					SelectObject(hdc, m_MessageBoxFont);
					SetTextColor(hdc, RGB(0, 0, 255));

					//
					// Other properties can be changed like this
					//
					// SetBkColor(hdc, RGB(218, 214, 192));
				}

				return true;
			}

			break;
		}
        case WM_QUERYNEWPALETTE:    // FALL THROUGH
        case WM_PALETTECHANGED:
        {
            //
            // invalidate to refresh palette
            //
            InvalidateRect(m_hwndMessageTextBox, NULL, FALSE);

			return 0;
        }
		case WM_APPCOMMAND:
        case WM_COMMAND:
        {
            //
            // This is a menu or accelerator method
            //
            if(	HIWORD(wParam) == 0 || HIWORD(wParam) == 1)
            {
				if (HandleMenuCommand(LOWORD(wParam)))
				{
					//
					// If we handled this command then return 0
					//
                    return 0;
				}
            }

			break;
        }
        case WM_SIZE:
        {
			//
			// Calculate size required for all controls
			//
			RECT rectClient;
            GetClientRect(inHWnd, &rectClient);

			int clientAreaHeight	= rectClient.bottom - rectClient.top;
			int clientAreaWidth		= rectClient.right - rectClient.left;
			
			RECT rectToolbar;
			GetClientRect(m_hwndToolBar, &rectToolbar);

			int toolbarHeight = rectToolbar.bottom - rectToolbar.top;

            RECT rectClientStatusBar;
            GetClientRect(m_hwndStatus, &rectClientStatusBar);

			int statusBarHeight = rectClientStatusBar.bottom - rectClientStatusBar.top;			

			//
			// HACK!
			// If i don't add this 2 here, the list control border gets hidden
			// by toolbar. Hence the HACK.
			//
			toolbarHeight += 2;

			int treeCtrlHeight		= ((clientAreaHeight - toolbarHeight - statusBarHeight) * 75) / 100;
			int treeCtrlWidth		= (clientAreaWidth * 25) / 100;

			int userListCtrlHeight	= clientAreaHeight - toolbarHeight - statusBarHeight - treeCtrlHeight;
			int userListCtrlWidth	= treeCtrlWidth;

			int listControlHeight	= (((clientAreaHeight - toolbarHeight - statusBarHeight) * 30) / 100);
			int listControlWidth	= clientAreaWidth - treeCtrlWidth;

			int textBoxHeight		= clientAreaHeight - toolbarHeight - statusBarHeight - listControlHeight;
			int textBoxWidth		= clientAreaWidth - treeCtrlWidth;

			//
			// Set position for all the controls
			//
			SetWindowPos(	m_hwndToolBar,
							NULL,
							0,
							0,
							rectClient.right - rectClient.left,
							toolbarHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			//
            // SetWindowPos does not work for status bar and this is
            // the method for its resizing
            //
            SendMessage(    m_hwndStatus,
                            WM_SIZE,
                            wParam,
                            lParam);

			SetWindowPos(	m_hwndUserTree,
							NULL,
							0,
							toolbarHeight,
							treeCtrlWidth,
							treeCtrlHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			SetWindowPos(	m_hwndUserList,
							NULL,
							0,
							toolbarHeight + treeCtrlHeight - 1, // - 1 to avoid double border
							userListCtrlWidth,
							userListCtrlHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);
			
            SetWindowPos(	m_hwndMessageList,
							NULL,
							treeCtrlWidth - 1,	// - 1 to avoid double border
							toolbarHeight,
							listControlWidth + 1,
							listControlHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			SetWindowPos(	m_hwndMessageTextBox,
							NULL,
							treeCtrlWidth,
							listControlHeight + toolbarHeight,
							textBoxWidth,
							textBoxHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			InvalidateRect(m_hwndMessageTextBox, NULL, TRUE);
			InvalidateRect(m_hwndMessageList, NULL, TRUE);
			InvalidateRect(m_hwndUserTree, NULL, TRUE);
			UpdateWindow(m_hwndMessageTextBox);
			UpdateWindow(m_hwndMessageList);
			UpdateWindow(m_hwndUserTree);

			break;
        }
        case WM_CHAR:
        {
            switch (wParam) 
            { 
                //
                // Handling of escape key.
				//
                case 0x1B:
                {                    
					return 0;
                }
            }

            break;
        }
		case WM_CREATE:
		{
			return 0;
		}
		case WM_CLOSE:
        {
			ExitInstance();
            return 0;
        }
		case WM_TIMER:
		{
			break;
		}
        default:
		{
            break;
		}
    }
    
	return DefWindowProc(inHWnd, nMsg, wParam, lParam);
}



/*++

Routine Name:
	
	HandleMenuCommand

Routine Description:
	
	Handles all the menu command for this windows menu

Returns:

	None

--*/
bool
CYArchDecode::HandleMenuCommand(
	WORD	inMenuCmd)
{
	bool bHandled = true;

    switch(inMenuCmd)
    {
		case ID_FILE_OPEN:
		{
			TCHAR tempFileName[MAX_PATH];
			tempFileName[0] = 0;

			OPENFILENAME ofn = {0};

			ofn.lStructSize		= OPENFILENAME_SIZE_VERSION_400;
			ofn.hwndOwner		= m_hwnd;
			ofn.hInstance		= m_hInstance;
			ofn.lpstrFilter		= _T("Archive Files (*.dat)\0*.dat;\0All Files (*.*)\0*.*\0\0");
			ofn.lpstrFile		= tempFileName;
			ofn.nMaxFile		= MAX_PATH;
			ofn.lpstrTitle		= _T("Open a Yahoo message archive file");
			ofn.Flags			= OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_PATHMUSTEXIST;

			if (GetOpenFileName(&ofn))
			{
				tstring profileId;
				tstring userName;
				tstring peerName;
				tstring fileName;

				fileName = ofn.lpstrFile;

				

				int nPos[5];

				nPos[0] = nPos[1] = nPos[2] = nPos[3] = nPos[4] = -1;

				int slashCount = 0;

				for (	int n = fileName.length() - 1;
						n >= 0;
						--n)
				{
					if (fileName[n] == L'\\')
					{
						nPos[slashCount++] = n;

						if (slashCount == 5)
						{
							break;
						}
					}
				}

				if (nPos[0] != -1)
				{
					profileId = fileName.substr(nPos[0] + 1, fileName.length() - nPos[1] - 1);
				}

				if (profileId.length() > 13)
				{
					profileId = profileId.substr(9, fileName.length() - 9 - 4);
				}

				if (nPos[1] != -1)
				{
					peerName = fileName.substr(nPos[1] + 1, nPos[0] - nPos[1] - 1);
				}

				if (nPos[4] != -1)
				{
					userName = fileName.substr(nPos[4] + 1, nPos[3] - nPos[4] - 1);
				}

				ARCHIVE_FILE_DATA archiveFileData;
				archiveFileData.archiveFile = fileName;
				archiveFileData.userName	= userName;
				archiveFileData.peerName	= peerName;
				archiveFileData.profileName	= profileId;

				DialogBoxParam(
					m_hInstance,
					MAKEINTRESOURCE(IDD_DIALOG_USERINFO),
					m_hwnd,
					(DLGPROC)UserInfoDlgProc,
					(LPARAM)&archiveFileData);

				PopulateTextBoxData(archiveFileData);
			}
			break;
		}
		case ID_FILE_SAVE:
		{
			if (gMsgText.length() > 0)
			{
				TCHAR fileName[MAX_PATH];
				fileName[0] = 0;

				OPENFILENAME sfn = {0};

				sfn.lStructSize		= OPENFILENAME_SIZE_VERSION_400;
				sfn.hwndOwner		= m_hwnd;
				sfn.hInstance		= m_hInstance;
				sfn.lpstrFile		= fileName;
				sfn.nMaxFile		= MAX_PATH;
				sfn.lpstrTitle		= _T("Save decoded Yahoo message archive to a file");
				sfn.Flags			= OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_OVERWRITEPROMPT;

				if (m_RichEdit)
				{
					sfn.lpstrFilter		= _T("RTF Files (*.rtf)\0*.rtf;\0\0");
					sfn.lpstrDefExt		= _T("rtf");
				}
				else
				{
					sfn.lpstrFilter		= _T("Text Files (*.txt)\0*.txt;\0\0");
					sfn.lpstrDefExt		= _T("txt");
				}

				if (GetSaveFileName(&sfn))
				{
					tstring fileName = sfn.lpstrFile;

					HANDLE hFile = CreateFile(
											sfn.lpstrFile,
											GENERIC_READ | GENERIC_WRITE,
											0,
											NULL,
											CREATE_ALWAYS,
											FILE_ATTRIBUTE_NORMAL,
											NULL);

					if (hFile != INVALID_HANDLE_VALUE)
					{
						DWORD bytesWritten = 0;

						WriteFile(
								hFile,
								gMsgText.c_str(),
								gMsgText.length() * sizeof(TCHAR),
								&bytesWritten,
								NULL);

						CloseHandle(hFile);
					}
					else
					{
						MessageBox(m_hwnd, _T("Error"), _T("Failed to create the specified file."), MB_OK);
					}
				}
			}

			break;
		}
        case ID_FILE_EXIT:
        {
			ExitInstance();
			break;
        }
		case ID_EDIT_COPY:
		{
			SendMessage(m_hwndMessageTextBox, WM_COPY, 0, 0);
			break;
		}
		case ID_EDIT_SELECTALL:
		{
			LONG lSel = (LONG) SendMessage (m_hwndMessageTextBox, WM_GETTEXTLENGTH, 0, 0L);
            SendMessage (m_hwndMessageTextBox, EM_SETSEL, 0, lSel );
            SendMessage(m_hwndMessageTextBox, EM_SCROLLCARET, 0, 0);
			SetFocus(m_hwndMessageTextBox);
			break;
		}
		case ID_VIEW_SHOWTIMESTAMP:
		{
			m_ShowTimestamp = !m_ShowTimestamp;

			CheckMenuItem(
					GetMenu(m_hwnd),
					ID_VIEW_SHOWTIMESTAMP,
					(m_ShowTimestamp ? MF_CHECKED : MF_UNCHECKED));
			int nSelectedItem = ListView_GetSelectionMark(m_hwndMessageList);

			if (nSelectedItem >= 0)
			{
				LVITEM lvItem	= {0};
				lvItem.mask		= LVIF_PARAM;
				lvItem.iItem	= nSelectedItem;
				
				if (ListView_GetItem(m_hwndMessageList, &lvItem))
				{
					int itemIndex = (int)lvItem.lParam;
					if (itemIndex > 0)
					{	
						ARCHIVE_FILE_DATA &archiveFileData = m_MessageListNodeParams[itemIndex - 1];
						PopulateTextBoxData(archiveFileData);
					}
				}
				else
				{
					nSelectedItem = GetLastError();
				}
			}
			break;
		}
		case ID_VIEW_REFRESH:
		{
			Refresh();
			break;
		}
		case ID_HELP_ABOUT:
		{
			DialogBox(
				m_hInstance,
				MAKEINTRESOURCE(IDD_DIALOG_ABOUT),
				m_hwnd,
				(DLGPROC)AboutDlgProc);

			break;
		}
        default:
		{
            bHandled = false;
            break;
		}
    }

	return bHandled;
}



/*++

Routine Name:
	
	CreateStatusBar

Routine Description:
	
	Creates an application specific status bar

Returns:

	<XXX>

--*/
int
CYArchDecode::CreateStatusBar()
{
    RECT rectClient;
    int lpParts[STATUS_BAR_NUM_PARTS];
    int nWidth;

    //
    // Create status bar window
    //
    m_hwndStatus = CreateWindowEx( 
                    0,                          // no extended styles
                    STATUSCLASSNAME,            // name of status bar class
                    (LPCTSTR) NULL,             // no text when first created
                    SBARS_SIZEGRIP |            // includes a sizing grip
                    WS_CHILD | WS_VISIBLE,      // creates a child window
                    0, 0, 0, 0,                 // ignores size and position
                    m_hwnd,                     // handle to parent window
                    (HMENU)IDC_CHILD_STATUS_BAR,// child window identifier
                    m_hInstance,                // handle to application instance
                    NULL);                      // no window creation data


    GetClientRect(m_hwnd, &rectClient);

    //
    // calculate the right edge coordinate for each part, and
    // copy the coordinates to the array.
    //
    nWidth = STATUS_WIDTH_OF_FIRST_PART;
	lpParts[0] = nWidth;

	nWidth += STATUS_WIDTH_OF_SECOND_PART;
	lpParts[1] = nWidth;

	nWidth += STATUS_WIDTH_OF_THIRD_PART;
	lpParts[2] = nWidth;

	nWidth += STATUS_WIDTH_OF_FOURTH_PART;
	lpParts[3] = nWidth;

    //
	// Tell the status bar to create the window parts.
    //
    SendMessage(
			m_hwndStatus,
            SB_SETPARTS,
			(WPARAM)STATUS_BAR_NUM_PARTS,
			(LPARAM)lpParts);

    SendMessage(
			m_hwndStatus,
            SB_SETTEXT,
            (WPARAM)0,
			(LPARAM)_T("Copyright (c) :"));

	SendMessage(
			m_hwndStatus,
            SB_SETTEXT,
            (WPARAM)1,
			(LPARAM)_T("Pankaj Garg"));

	SendMessage(
			m_hwndStatus,
			SB_SETTEXT,
			(WPARAM)2,
			(LPARAM)_T("http://www.intellectualheaven.com"));

	SendMessage(
			m_hwndStatus,
			SB_SETTEXT,
			(WPARAM)3,
			(LPARAM)_T("All rights reserved."));

    return 0;
}



/*++

Routine Name:
	
	AddToolbarButton

Routine Description:
	
	Adds a button to the toolbar

Returns:

	None

--*/
void
CYArchDecode::AddToolbarButton(
	int			iBitmap,
	int			idCommand,
	BYTE		fsState,
	BYTE		fsStyle,
	DWORD_PTR	dwData,
	INT_PTR		iString)
{
	SendMessage(m_hwndToolBar, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0); 

	TBBUTTON tbb;

	tbb.iBitmap		= iBitmap;
	tbb.idCommand	= idCommand;
	tbb.fsState		= fsState;
	tbb.fsStyle		= fsStyle;
	tbb.dwData		= dwData; 
	tbb.iString		= iString;

	SendMessage(m_hwndToolBar, TB_ADDBUTTONS, (WPARAM)1, (LPARAM)(LPTBBUTTON)&tbb);
	SendMessage(m_hwndToolBar, TB_AUTOSIZE, 0, 0);
}



/*++

Routine Name:
	
	CreateToolBar

Routine Description:
	
	Creates a toolbar for the application

Returns:

	None

--*/
void
CYArchDecode::CreateToolBar() 
{ 
	TBADDBITMAP				tbab;

	//
	// Create a toolbar
	//
	m_hwndToolBar = CreateWindowEx(	
								0,
								TOOLBARCLASSNAME,
								(LPTSTR)NULL,
								WS_CHILD | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS | (m_OldComctl32 ? 0 : TBSTYLE_LIST),
								0, 0, 0, 0,
								m_hwnd,
								(HMENU)IDC_CHILD_TOOL_BAR,
								m_hInstance,
								NULL);

	//
	// Set toolbar style
	//
	SendMessage(m_hwndToolBar, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_MIXEDBUTTONS);

	//
	// Big Note:
	// I couldn't find a better way to load images exported by shell32 for toolbar
	// icons and also have my images and create an image list. So i first add
	// shell32 exported bitmap images to the toolbar, then i copy this image
	// list to a two different image list for normal and hot tracking of items
	// and then add custom bitmaps to it. This way i can use shell32 bitmaps as
	// well as custom bitmaps and also can have hot tracking.
	//
	tbab.hInst	= HINST_COMMCTRL;
	tbab.nID	= IDB_VIEW_LARGE_COLOR;

	int viewLargeImageIndex = SendMessage(m_hwndToolBar, TB_ADDBITMAP, (WPARAM)1, (LPARAM)(LPTBBUTTON)&tbab);

	tbab.hInst	= HINST_COMMCTRL;
	tbab.nID	= IDB_STD_LARGE_COLOR;

	int stdLargeImageIndex = SendMessage(m_hwndToolBar, TB_ADDBITMAP, (WPARAM)1, (LPARAM)(LPTBBUTTON)&tbab);

	HIMAGELIST himlTemp = (HIMAGELIST)SendMessage(m_hwndToolBar, TB_GETIMAGELIST, (WPARAM)0, (LPARAM)0);

	HIMAGELIST himlNormal	= ImageList_Duplicate(himlTemp);
	HIMAGELIST himlHot		= ImageList_Duplicate(himlTemp);

	HBITMAP hBmp;
	
	hBmp = (HBITMAP)LoadBitmap(m_hInstance, MAKEINTRESOURCE(IDB_GREEN_OPTION));
	int optionImageIndex = ImageList_Add(himlNormal, hBmp, NULL);
	DeleteObject(hBmp);

	hBmp = (HBITMAP)LoadBitmap(m_hInstance, MAKEINTRESOURCE(IDB_GREEN_ABOUT));
	int aboutImageIndex = ImageList_Add(himlNormal, hBmp, NULL);
	DeleteObject(hBmp);

	hBmp = (HBITMAP)LoadBitmap(m_hInstance, MAKEINTRESOURCE(IDB_ORANGE_OPTION));
	ImageList_Add(himlHot, hBmp, NULL);
	DeleteObject(hBmp);

	hBmp = (HBITMAP)LoadBitmap(m_hInstance, MAKEINTRESOURCE(IDB_ORANGE_ABOUT));
	ImageList_Add(himlHot, hBmp, NULL);
	DeleteObject(hBmp);

	SendMessage(m_hwndToolBar, TB_SETIMAGELIST, (WPARAM)0, (LPARAM)himlNormal);
	SendMessage(m_hwndToolBar, TB_SETHOTIMAGELIST, (WPARAM)0, (LPARAM)himlHot);

	//
	// Destroy the temporary list
	//
	ImageList_Destroy(himlTemp);

	//
	// Add required buttons to the toolbar
	//
	AddToolbarButton(
		STD_FILEOPEN + stdLargeImageIndex,
		ID_FILE_OPEN,
		TBSTATE_ENABLED,
		BTNS_BUTTON,
		0,
		(m_OldComctl32 ? -1 : (INT_PTR)_T("Open an archive file")));

	AddToolbarButton(
		STD_FILESAVE + stdLargeImageIndex,
		ID_FILE_SAVE,
		TBSTATE_ENABLED,
		BTNS_BUTTON,
		0,
		(m_OldComctl32 ? -1 : (INT_PTR)_T("Save archive file")));

	AddToolbarButton(
		I_IMAGENONE,
		0,
		TBSTATE_ENABLED,
		BTNS_SEP,
		0,
		-1);

	AddToolbarButton(
		STD_COPY + stdLargeImageIndex,
		ID_EDIT_COPY,
		TBSTATE_ENABLED,
		BTNS_BUTTON,
		0,
		(m_OldComctl32 ? -1 : (INT_PTR)_T("Copies the decoded message text")));

	AddToolbarButton(
		I_IMAGENONE,
		0,
		TBSTATE_ENABLED,
		BTNS_SEP,
		0,
		-1);

	AddToolbarButton(
		VIEW_DETAILS + viewLargeImageIndex,
		ID_VIEW_REFRESH,
		TBSTATE_ENABLED,
		BTNS_BUTTON,
		0,
		(m_OldComctl32 ? -1 : (INT_PTR)_T("Refresh")));

	AddToolbarButton(
		I_IMAGENONE,
		0,
		TBSTATE_ENABLED,
		BTNS_SEP,
		0,
		-1);

	AddToolbarButton(
		aboutImageIndex,
		ID_HELP_ABOUT,
		TBSTATE_ENABLED,
		BTNS_BUTTON,
		0,
		(m_OldComctl32 ? -1 : (INT_PTR)_T("About YahDecode")));

	ShowWindow(m_hwndToolBar, SW_SHOW);
}



/*++

Routine Name:
	
	CreateUserTreeControl

Routine Description:
	
	Creates a tree control to show the list of users and
	their archive conference and message list.

Returns:

	None

--*/
void
CYArchDecode::CreateUserTreeControl()
{
    //
    // create and fill list view
    //
    m_hwndUserTree = CreateWindowEx(
								0,
                                WC_TREEVIEW,
                                NULL,
                                WS_CHILD | WS_VISIBLE | WS_BORDER |
								TVS_HASLINES | TVS_HASBUTTONS | TVS_SHOWSELALWAYS | TVS_LINESATROOT,
                                0, 0, 0, 0,
                                m_hwnd,
                                (HMENU)IDC_CHILD_TREE_VIEW,
                                m_hInstance,
                                NULL);

    if (m_hwndUserTree == NULL)
    {
		//
		// To-Do!!!
		// Error handling
		//
    }
}



/*++

Routine Name:
	
	AttachImageListToTree

Routine Description:
	
	Creates an Images list and attach it to the user
	tree control.

Returns:

	None

--*/
void 
CYArchDecode::AttachImageListToTree()
{
    HBITMAP hBitmap;

    //
    // Create image lists as dib section 24-bit color
    // large image list
    //
    m_hILUserTree = ImageList_Create(
								16, 16,
								ILC_COLOR24 | ILC_MASK,
								9, 1);

    hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_USER_MAIN),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

    m_UserMainImageIndex = 
		ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,0,0));

    DeleteObject(hBitmap);


	hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_MESSAGE),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

	m_MessageImageIndex =
		ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,255,255));

    DeleteObject(hBitmap);


	hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_ALERT),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

	m_AlertImageIndex =
		ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,255,255));

    DeleteObject(hBitmap);


	hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_CONFERENCE),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

	m_ConferenceImageIndex =
		ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,255,255));

    DeleteObject(hBitmap);


	hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_MOBILE),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

	m_MobileImageIndex =
    ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,255,255));

    DeleteObject(hBitmap);


	hBitmap = (HBITMAP)LoadImage(
							m_hInstance,
							MAKEINTRESOURCE(IDB_USER_SINGLE),
							IMAGE_BITMAP,
							0, 0,
							LR_CREATEDIBSECTION);

	m_UserSingleImageIndex =
		ImageList_AddMasked(
					m_hILUserTree,
					hBitmap,
					RGB(0,255,255));

    DeleteObject(hBitmap);

    

    // 
    // Assign image lists to the control
    //
    TreeView_SetImageList(
					m_hwndUserTree,
					m_hILUserTree,
					LVSIL_NORMAL);
}



/*++

Routine Name:
	
	PopulateUserTree

Routine Description:
	
	Populates the user tree control with all the user names
	and their archive messages

Returns:

	None

--*/
void
CYArchDecode::PopulateUserTree()
{	
	//
	// Clear all the tree related controls
	//
	ListView_DeleteAllItems(m_hwndMessageList);
	m_MessageListNodeParams.clear();
	SetMessageTextBoxText(_T(""));
	TreeView_DeleteAllItems(m_hwndUserTree);
	m_TreeNodeParams.clear();

	DWORD driveList = GetLogicalDrives();

	int driveIndex = 0;

	for (int i = 0; i < 26; ++i)
	{
		driveIndex = 1 << i;

		if (driveIndex & driveList)
		{
			TCHAR ch[2] = {0};
			ch[0] = _T('A') + i;
			tstring driveLetter = ch;
			driveLetter = driveLetter + _T(":\\");
			if (GetDriveType(driveLetter.c_str()) == DRIVE_FIXED)
			{
				//
				// Brute-Force yahoo archive path detector
				//

				//
				// First Check the folders <drive>\Program Files\Yahoo!\Messenger\Profiles
				// If this folder exist, we look for user name which might have some
				// archived messages
				//
				tstring yahooInstallPath = driveLetter + _T("Program Files\\Yahoo!\\Messenger\\Profiles");
				AddAllUsersArchivesToTree(yahooInstallPath);

				// For Windows 2000 onwards
				tstring winUserProfilePath = driveLetter + _T("Documents and Settings");
				TravelWinUserProfilesForYahooUsersArchive(winUserProfilePath);

				// For Windows 9x and ME
				winUserProfilePath = driveLetter + _T("Windows\\Profiles");
				TravelWinUserProfilesForYahooUsersArchive(winUserProfilePath);

				// For Windows NT 4.0 with SP6
				winUserProfilePath = driveLetter + _T("Winnt\\Profiles");
				TravelWinUserProfilesForYahooUsersArchive(winUserProfilePath);				
			}
		}
	}
}


/*++

Routine Name:
	
	TravelWinUserProfilesForYahooUsersArchive

Routine Description:
	
	Traverse each possible Windows user profile directory
	to see if there is a Yahoo! profile folder inside it.
	The Yahoo profile folder sometime contains Archive folder
	if its not able to create the archive in yahoo Install
	directory.

Returns:

	None

--*/
void
CYArchDecode::TravelWinUserProfilesForYahooUsersArchive(
	tstring &winUserProfileFolder)
{
	DWORD fileAttr = GetFileAttributes(winUserProfileFolder.c_str());
	if (fileAttr & FILE_ATTRIBUTE_DIRECTORY)
	{
		tstring winUserProfilePath = winUserProfileFolder + _T("\\*");
		WIN32_FIND_DATA fileFindData;
		HANDLE hFindFileRoot = ::FindFirstFile(
										winUserProfilePath.c_str(),
										&fileFindData);

		if (hFindFileRoot != INVALID_HANDLE_VALUE)
		{
			do
			{
				if (fileFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if (_tcsicmp(fileFindData.cFileName, _T(".")) != 0 &&
						_tcsicmp(fileFindData.cFileName, _T("..")) != 0)
					{
						tstring yahooUsersProfileFolder = winUserProfileFolder + _T("\\") + fileFindData.cFileName + _T("\\Application Data\\Yahoo!\\Messenger\\Profiles") ;
						AddAllUsersArchivesToTree(yahooUsersProfileFolder);
					}
				}
			} while(::FindNextFile(hFindFileRoot, &fileFindData));
		}
	}
}


/*++

Routine Name:
	
	AddAllUsersArchivesToTree

Routine Description:
	
	Once we have a profile path, this function goes in through
	all user profiles of yahoo and populates items in tree one
	by one

Returns:

	None

--*/
void
CYArchDecode::AddAllUsersArchivesToTree(
	tstring &dirPath)
{
	DWORD fileAttr = GetFileAttributes(dirPath.c_str());
	if (fileAttr & FILE_ATTRIBUTE_DIRECTORY)
	{
		tstring profilePath = dirPath + _T("\\*");
		WIN32_FIND_DATA fileFindData;
		HANDLE hFindFileRoot = ::FindFirstFile(
										profilePath.c_str(),
										&fileFindData);

		if (hFindFileRoot != INVALID_HANDLE_VALUE)
		{
			do
			{
				if (fileFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if (_tcsicmp(fileFindData.cFileName, _T(".")) != 0 &&
						_tcsicmp(fileFindData.cFileName, _T("..")) != 0)
					{
						tstring userProfilePath = dirPath + _T("\\") + fileFindData.cFileName;
						tstring userName		= fileFindData.cFileName;
						AddUserArchivesToTree(userName, userProfilePath);
					}
				}
			} while(::FindNextFile(hFindFileRoot, &fileFindData));
		}
	}
}


/*++

Routine Name:
	
	AddUserArchivesToTree

Routine Description:
	
	Called from AddAllUsersArchivesToTree to populate each
	user's information in tree one by one

Returns:

	None

--*/
void
CYArchDecode::AddUserArchivesToTree(
		tstring &userName, 
		tstring &userProfilePath)
{	
	TVINSERTSTRUCT	tvis = {0};

	HTREEITEM		hRootUserItem;
	HTREEITEM		hMessagesItem;
	HTREEITEM		hAlertsItem;
	HTREEITEM		hConferenceItem;
	HTREEITEM		hMobileItem;

	tstring nodeTitle;

	nodeTitle					= userName;
	tvis.hInsertAfter			= TVI_SORT;
	tvis.item.mask				= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	tvis.item.cChildren			= 1;
	tvis.item.pszText			= (LPTSTR)nodeTitle.c_str();
	tvis.item.cchTextMax		= nodeTitle.length();
	tvis.item.iImage			= m_UserMainImageIndex;
	tvis.item.iSelectedImage	= m_UserMainImageIndex;

	hRootUserItem = TreeView_InsertItem(m_hwndUserTree, &tvis);

	tstring archivePath			= userProfilePath + _T("\\Archive");	
	tstring messageArchivePath	= archivePath + _T("\\Messages");
	tstring alertArchivePath	= archivePath + _T("\\Alerts");
	tstring confArchivePath		= archivePath + _T("\\Conferences");
	tstring mobileArchivePath	= archivePath + _T("\\Mobile Messages");

	//
	// Now Add all the archive folders like Messages etc
	// All will be added directly under Root user item
	//
	tvis.hParent				= hRootUserItem;
	tvis.hInsertAfter			= TVI_LAST;
	tvis.item.mask				= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;

	ARCHIVE_TREE_DATA archiveTreeData;
	archiveTreeData.nodeType		= NODE_CATEGORY;
	archiveTreeData.userName		= userName;
	archiveTreeData.peerName		= _T("");

	archiveTreeData.archivePath		= messageArchivePath;
	m_TreeNodeParams.push_back(archiveTreeData);
	int itemIndex = m_TreeNodeParams.size();

	nodeTitle					= _T("Messages");
	tvis.item.pszText			= (LPTSTR)nodeTitle.c_str();
	tvis.item.cchTextMax		= nodeTitle.length();
	tvis.item.iImage			= m_MessageImageIndex;
	tvis.item.iSelectedImage	= m_MessageImageIndex;
	tvis.item.lParam			= itemIndex;
	hMessagesItem = TreeView_InsertItem(m_hwndUserTree, &tvis);

	archiveTreeData.archivePath		= alertArchivePath;
	m_TreeNodeParams.push_back(archiveTreeData);
	itemIndex = m_TreeNodeParams.size();

	nodeTitle					= _T("Alerts");
	tvis.item.pszText			= (LPTSTR)nodeTitle.c_str();
	tvis.item.cchTextMax		= nodeTitle.length();
	tvis.item.iImage			= m_AlertImageIndex;
	tvis.item.iSelectedImage	= m_AlertImageIndex;
	tvis.item.lParam			= itemIndex;
	hAlertsItem = TreeView_InsertItem(m_hwndUserTree, &tvis);

	archiveTreeData.archivePath		= confArchivePath;
	m_TreeNodeParams.push_back(archiveTreeData);
	itemIndex = m_TreeNodeParams.size();

	nodeTitle					= _T("Conferences");
	tvis.item.pszText			= (LPTSTR)nodeTitle.c_str();
	tvis.item.cchTextMax		= nodeTitle.length();
	tvis.item.iImage			= m_ConferenceImageIndex;
	tvis.item.iSelectedImage	= m_ConferenceImageIndex;
	tvis.item.lParam			= itemIndex;
	hConferenceItem = TreeView_InsertItem(m_hwndUserTree, &tvis);

	archiveTreeData.archivePath		= mobileArchivePath;
	m_TreeNodeParams.push_back(archiveTreeData);
	itemIndex = m_TreeNodeParams.size();

	nodeTitle					= _T("Mobile Messages");
	tvis.item.pszText			= (LPTSTR)nodeTitle.c_str();
	tvis.item.cchTextMax		= nodeTitle.length();
	tvis.item.iImage			= m_MobileImageIndex;
	tvis.item.iSelectedImage	= m_MobileImageIndex;
	tvis.item.lParam			= itemIndex;
	hMobileItem = TreeView_InsertItem(m_hwndUserTree, &tvis);

	//
	// Now traverse these folders one by one and add if there
	// are any messages in there
	//
	
	AddCategorizedUserArchiveToTree(userName, hMessagesItem, messageArchivePath);
	AddCategorizedUserArchiveToTree(userName, hAlertsItem, alertArchivePath);	
	AddCategorizedUserArchiveToTree(userName, hConferenceItem, confArchivePath);
	AddCategorizedUserArchiveToTree(userName, hMobileItem, mobileArchivePath);
}


/*++

Routine Name:
	
	AddCategorizedUserArchiveToTree

Routine Description:
	
	Called to add different types of archive to the tree
	for a given user. The method to add the archive
	is essentially the same. The four archives known to us
	are Messages, Mobile Messages, Conferences and Alerts

Returns:

	None

--*/
void
CYArchDecode::AddCategorizedUserArchiveToTree(
	tstring		&userName,
	HTREEITEM	&hItem,
	tstring		&archivePath)
{
	tstring searchArchivePath = archivePath + _T("\\*");
	WIN32_FIND_DATA fileFindData;
	HANDLE hFindFileRoot = ::FindFirstFile(
									searchArchivePath.c_str(),
									&fileFindData);

	if (hFindFileRoot != INVALID_HANDLE_VALUE)
	{
		do
		{
			if (fileFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if (_tcsicmp(fileFindData.cFileName, _T(".")) != 0 &&
					_tcsicmp(fileFindData.cFileName, _T("..")) != 0)
				{
					tstring peerName = fileFindData.cFileName;

					ARCHIVE_TREE_DATA archiveTreeData;
					archiveTreeData.nodeType		= NODE_LEAF_USER;
					archiveTreeData.userName		= userName;
					archiveTreeData.peerName		= peerName;
					archiveTreeData.archivePath		= archivePath;
					m_TreeNodeParams.push_back(archiveTreeData);
					int itemIndex = m_TreeNodeParams.size();

					TVINSERTSTRUCT	tvis = {0};
					tvis.item.mask				= TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
					tvis.hParent				= hItem;
					tvis.hInsertAfter			= TVI_SORT;
					tvis.item.pszText			= (LPTSTR)peerName.c_str();
					tvis.item.cchTextMax		= peerName.length();
					tvis.item.iImage			= m_UserSingleImageIndex;
					tvis.item.iSelectedImage	= m_UserSingleImageIndex;
					tvis.item.lParam			= itemIndex;
					TreeView_InsertItem(m_hwndUserTree, &tvis);
				}
			}
		} while(::FindNextFile(hFindFileRoot, &fileFindData));
	}
}


/*++

Routine Name:
	
	CreateUserListControl

Routine Description:
	
	Creates a list control for users and their archive settings.

Returns:

	None

--*/
void
CYArchDecode::CreateUserListControl()
{
    //
    // create and fill list view
    //
    m_hwndUserList = CreateWindowEx(
								0,
                                WC_LISTVIEW,
                                NULL,
                                WS_CHILD | WS_VISIBLE | WS_BORDER |
								LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL | LVS_SORTASCENDING  | LVS_NOSORTHEADER,
                                0, 0, 0, 0,
                                m_hwnd,
                                (HMENU)IDC_CHILD_USER_LIST,
                                m_hInstance,
                                NULL);

    if (m_hwndMessageList)
    {
        ListView_SetExtendedListViewStyle(  m_hwndUserList,
                                            LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES);
		

        //
        // Create Column
        //
        LVCOLUMN lvColumn;
        lvColumn.mask = LVCF_TEXT | LVCF_WIDTH;

        //
        // Insert columns in main List control
        //
		for (	int listHeaderItem = 0;
				listHeaderItem < (sizeof(gUserListControlData) / sizeof(gUserListControlData[0]));
				++listHeaderItem)
		{
			lvColumn.cx			= gUserListControlData[listHeaderItem].width;
			lvColumn.pszText	= gUserListControlData[listHeaderItem].title;

			ListView_InsertColumn(  m_hwndUserList,
									listHeaderItem,
									&lvColumn);
		}
    }
}



/*++

Routine Name:
	
	PopulateUserListControl

Routine Description:
	
	This function goes through the registry and traverse
	all the yahoo users in the current windows profile.
	It display each user name and whether yahoo archiving
	is enabled for them or not in a list control

Returns:

	None

--*/
void
CYArchDecode::PopulateUserListControl()
{
	ListView_DeleteAllItems(m_hwndUserList);

	HKEY hInfoRootKey = NULL;

	int nRetVal = 0;

	nRetVal = RegOpenKey(
					HKEY_CURRENT_USER,
					YAHOO_INFO_REG_KEY,
					&hInfoRootKey);

	if (nRetVal != ERROR_SUCCESS)
	{
		goto funcExit;
	}

	DWORD regIndex = 0;
	TCHAR userName[MAX_YAHOO_USER_NAME_LEN + 1] = {0};

	while(ERROR_SUCCESS == RegEnumKey(
								hInfoRootKey,
								regIndex++,
								userName,
								MAX_YAHOO_USER_NAME_LEN))
	{
		tstring userKeyName = YAHOO_INFO_REG_KEY;
		userKeyName = userKeyName + _T("\\") + userName;

		HKEY hUserKey = NULL;

		TCHAR *temp = (TCHAR *)userKeyName.c_str();

		nRetVal = RegOpenKey(
						HKEY_CURRENT_USER,
						userKeyName.c_str(),
						&hUserKey);

		//
		// If user key doesn't exist that means RegEnumKey
		// Screwed up or someone deleted it behind out back
		//
		if (nRetVal != ERROR_SUCCESS)
		{
			continue;
		}

		DWORD dataSize = 0;
		nRetVal = RegQueryValueEx(
								hUserKey,
								_T("All Identities"),
								NULL,
								NULL,
								NULL,
								&dataSize);

		//
		// Close the key we don't need it anymore
		//
		RegCloseKey(hUserKey);

		//
		// All Identities tells us if this is really a
		// Yahoo Id or just a profile
		//
		if (nRetVal != ERROR_SUCCESS)
		{
			continue;
		}

		DWORD archiveEnabled = 0;


		//
		// Lets see if this user has any archived settings
		//
		tstring userArchKeyName = YAHOO_INFO_REG_KEY; 
		userArchKeyName = userArchKeyName + _T("\\") + userName + _T("\\Archive");

		HKEY hUserArchKey = NULL;
		nRetVal = RegOpenKey(
						HKEY_CURRENT_USER,
						userArchKeyName.c_str(),
						&hUserArchKey);

		if (nRetVal == ERROR_SUCCESS)
		{		
			dataSize = sizeof(archiveEnabled);
			nRetVal = RegQueryValueEx(
									hUserArchKey,
									_T("Enabled"),
									NULL,
									NULL,
									(LPBYTE)&archiveEnabled,
									&dataSize);

			//
			// Close the key we don't need it anymore
			//
			RegCloseKey(hUserArchKey);
		}
		
		//
		// Add user entry to the list box
		//
		LVITEM  lvItem;
		ZeroMemory(&lvItem, sizeof(LVITEM));

		lvItem.mask		= LVIF_TEXT | LVIF_PARAM;
		lvItem.iItem	= 0;
		lvItem.lParam	= 0;
		lvItem.iSubItem = 0;
		lvItem.pszText	= (LPTSTR)userName;

		int itemIndex =
		ListView_InsertItem(
						m_hwndUserList,
						&lvItem);

		if (itemIndex != -1)
		{
			if (archiveEnabled)
			{
				lvItem.pszText	= (LPTSTR)_T("Enabled");
				ListView_SetCheckState(m_hwndUserList, itemIndex, TRUE);
			}
			else
			{
				lvItem.pszText	= (LPTSTR)_T("Disabled");
				ListView_SetCheckState(m_hwndUserList, itemIndex, FALSE);
			}
			
			lvItem.iItem	= itemIndex;
			lvItem.mask		= LVIF_TEXT;	// remove LVIF_PARAM (can't set for subitem)

			lvItem.iSubItem = USER_LISTCTRL_ARCHIVING_POS;

			ListView_SetItem(
							m_hwndUserList,
							&lvItem);
			
		}
	}
	


funcExit:

	if (hInfoRootKey)
	{
		RegCloseKey(hInfoRootKey);
	}

	return;
}



/*++

Routine Name:
	
	EnableArchiveForYahooUser

Routine Description:
	
	Enable or Disable archiving for a particular
	Yahoo User.

Returns:

	None

--*/
void
CYArchDecode::EnableArchiveForYahooUser(
	LPTSTR	userName,
	DWORD	itemIndex,
	BOOL	shouldEnable)
{
	HKEY hInfoRootKey		= NULL;
	int nRetVal				= 0;
	tstring userArchKeyName;

	nRetVal = RegOpenKey(
					HKEY_CURRENT_USER,
					YAHOO_INFO_REG_KEY,
					&hInfoRootKey);

	if (nRetVal != ERROR_SUCCESS)
	{
		goto funcExit;
	}
	
	userArchKeyName = userArchKeyName + YAHOO_INFO_REG_KEY + _T("\\") + userName + _T("\\Archive");

	//
	// Try to open the user's archive key and if it doesn't exist
	// then try to create a new key for archive
	//
	HKEY hUserArchKey = NULL;
	nRetVal = RegOpenKey(
					HKEY_CURRENT_USER,
					userArchKeyName.c_str(),
					&hUserArchKey);

	if (nRetVal != ERROR_SUCCESS)
	{
		nRetVal = RegCreateKey(
						HKEY_CURRENT_USER,
						userArchKeyName.c_str(),
						&hUserArchKey);

		if (nRetVal != ERROR_SUCCESS)
		{
			goto funcExit;
		}
	}

	//
	// If we reach here it means yahoo user now has an archive
	// key. Its time to either enable/disable the archiving in
	// his profile
	//
	DWORD enableValue = (shouldEnable) ? 1 : 0;
	
	RegSetValueEx(
			hUserArchKey,
			_T("Enabled"),
			NULL,
			REG_DWORD,
			(const LPBYTE)&enableValue,
			sizeof(enableValue));

	DWORD daysDeleteAfter = 999;
	RegSetValueEx(
			hUserArchKey,
			_T("DeleteAfter"),
			NULL,
			REG_DWORD,
			(const LPBYTE)&daysDeleteAfter,
			sizeof(daysDeleteAfter));

	BYTE archSettings1[8] = {0xDF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
	RegSetValueEx(
			hUserArchKey,
			_T("ArchiveSettings1"),
			NULL,
			REG_BINARY,
			(const LPBYTE)&archSettings1,
			sizeof(archSettings1));


	//
	// Update the list control to show proper state enabled/disabled
	//
	LVITEM  lvItem;
	ZeroMemory(&lvItem, sizeof(LVITEM));

	lvItem.mask		= LVIF_TEXT;
	lvItem.iItem	= itemIndex;
	lvItem.iSubItem = USER_LISTCTRL_ARCHIVING_POS;
	
	if (shouldEnable)
	{
		lvItem.pszText	= (LPTSTR)_T("Enabled");
	}
	else
	{
		lvItem.pszText	= (LPTSTR)_T("Disabled");
	}

	ListView_SetItem(
				m_hwndUserList,
				&lvItem);


funcExit:

	if (hUserArchKey)
	{
		RegCloseKey(hUserArchKey);
	}

	if (hInfoRootKey)
	{
		RegCloseKey(hInfoRootKey);
	}

	return;
}



/*++

Routine Name:
	
	CreateMessageListControl

Routine Description:
	
	Creates a list control for messages or conference 
	from one user.

Returns:

	None

--*/
void
CYArchDecode::CreateMessageListControl()
{
    //
    // create and fill list view
    //
    m_hwndMessageList = CreateWindowEx(
								0,
                                WC_LISTVIEW,
                                NULL,
                                WS_CHILD | WS_VISIBLE | WS_BORDER |
								LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL | LVS_SORTDESCENDING | LVS_NOSORTHEADER,
                                0, 0, 0, 0,
                                m_hwnd,
                                (HMENU)IDC_CHILD_MESSAGE_LIST,
                                m_hInstance,
                                NULL);

    if (m_hwndMessageList)
    {
        ListView_SetExtendedListViewStyle(  m_hwndMessageList,
                                            LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

        //
        // Create Column
        //
        LVCOLUMN lvColumn;
        lvColumn.mask = LVCF_TEXT | LVCF_WIDTH;

        //
        // Insert columns in main List control
        //
		for (	int listHeaderItem = 0;
				listHeaderItem < (sizeof(gMsgListControlData) / sizeof(gMsgListControlData[0]));
				++listHeaderItem)
		{
			lvColumn.cx			= gMsgListControlData[listHeaderItem].width;
			lvColumn.pszText	= gMsgListControlData[listHeaderItem].title;

			ListView_InsertColumn(  m_hwndMessageList,
									listHeaderItem,
									&lvColumn);
		}

    }
}


/*++

Routine Name:
	
	PopulateMessageListCtrlWholeCategory

Routine Description:
	
	Add archive messages from a particular category like
	Messages, Conference etc for a user. This will add message
	from all the users from that user.

Returns:

	None

--*/
void
CYArchDecode::PopulateMessageListCtrlWholeCategory(
	ARCHIVE_TREE_DATA &archiveTreeData)
{
	tstring searchArchivePath = archiveTreeData.archivePath + _T("\\*");
	WIN32_FIND_DATA fileFindData;
	HANDLE hFindFileRoot = ::FindFirstFile(
									searchArchivePath.c_str(),
									&fileFindData);

	if (hFindFileRoot != INVALID_HANDLE_VALUE)
	{
		do
		{
			if (fileFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if (_tcsicmp(fileFindData.cFileName, _T(".")) != 0 &&
					_tcsicmp(fileFindData.cFileName, _T("..")) != 0)
				{
					tstring peerName = fileFindData.cFileName;

					ARCHIVE_TREE_DATA tempTreeData;
					tempTreeData.nodeType		= NODE_LEAF_USER;
					tempTreeData.userName		= archiveTreeData.userName;
					tempTreeData.peerName		= peerName;
					tempTreeData.archivePath	= archiveTreeData.archivePath;
					
					PopulateMessageListCtrl(tempTreeData);
				}
			}
		} while(::FindNextFile(hFindFileRoot, &fileFindData));
	}
}


/*++

Routine Name:
	
	PopulateMessageListCtrl

Routine Description:
	
	Add archive list for a user, from a particular user

Returns:

	None

--*/
void
CYArchDecode::PopulateMessageListCtrl(
	ARCHIVE_TREE_DATA &archiveTreeData)
{
	tstring searchArchivePath = archiveTreeData.archivePath + _T("\\") + archiveTreeData.peerName + _T("\\*.dat");
	
	WIN32_FIND_DATA fileFindData;
	HANDLE hFindFileRoot = ::FindFirstFile(
									searchArchivePath.c_str(),
									&fileFindData);

	if (hFindFileRoot != INVALID_HANDLE_VALUE)
	{
		do
		{
			if ((fileFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
			{
				tstring fileName	= fileFindData.cFileName;
				tstring fileDate	= fileName.substr(0, 4) + _T("/") + fileName.substr(4, 2) + _T("/") + fileName.substr(6, 2);
				tstring profileId	= fileName.substr(9, fileName.length() - 9 - 4);

				ARCHIVE_FILE_DATA archiveFileData;
				archiveFileData.userName		= archiveTreeData.userName;
				archiveFileData.profileName		= profileId;
				archiveFileData.peerName		= archiveTreeData.peerName;
				archiveFileData.archiveFile		= archiveTreeData.archivePath + _T("\\") + archiveTreeData.peerName + _T("\\") + fileFindData.cFileName;
				m_MessageListNodeParams.push_back(archiveFileData);

				int userDataIndex = m_MessageListNodeParams.size();

				LVITEM  lvItem;
				ZeroMemory(&lvItem, sizeof(LVITEM));

				lvItem.mask		= LVIF_TEXT | LVIF_PARAM;
				lvItem.iItem	= 0;
				lvItem.lParam	= userDataIndex;
				lvItem.iSubItem = LISTCTRL_DATE_POS;
				lvItem.pszText	= (LPTSTR)fileDate.c_str();

				int itemIndex =
				ListView_InsertItem(
								m_hwndMessageList,
								&lvItem);

				if (itemIndex != -1)
				{
					//
					// No param for sub-items
					//
					lvItem.iItem	= itemIndex;
					lvItem.mask		= LVIF_TEXT;

					lvItem.iSubItem = LISTCTRL_FRIEND_OR_HOST_POS;
					lvItem.pszText	= (LPTSTR)archiveTreeData.peerName.c_str();
					ListView_SetItem(
									m_hwndMessageList,
									&lvItem);

					lvItem.iSubItem = LISTCTRL_YOUR_PROFILE_POS;
					lvItem.pszText	= (LPTSTR)profileId.c_str();
					ListView_SetItem(
									m_hwndMessageList,
									&lvItem);
				}
			}
		} while(::FindNextFile(hFindFileRoot, &fileFindData));
	}
}



/*++

Routine Name:
	
	CreateMessageTextBox

Routine Description:
	
	Creates a text box to display message in one
	conversation from a user or conference

Returns:

	None

--*/
void
CYArchDecode::CreateMessageTextBox()
{
	LoadLibrary(_T("riched32.dll"));

	//
    // Read only edit box message text
    //
    m_hwndMessageTextBox = CreateWindowEx(	
									0,
									_T("RichEdit"),
									NULL,
									WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL |
									ES_MULTILINE | ES_READONLY,
									0, 0, 0, 0,
									m_hwnd,
									(HMENU)IDC_CHILD_EDIT_BOX,
									m_hInstance,
									NULL);

    if (m_hwndMessageTextBox)
    {
		m_RichEdit = true;
		SendMessage(
			m_hwndMessageTextBox,
			EM_SETBKGNDCOLOR,
			0,
			RGB(220, 220, 200));
    }
	else
	{
		m_RichEdit = false;

		m_hwndMessageTextBox = CreateWindowEx(	
									0,
									_T("Edit"),
									NULL,
									WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL |
									ES_MULTILINE | ES_READONLY,
									0, 0, 0, 0,
									m_hwnd,
									(HMENU)IDC_CHILD_EDIT_BOX,
									m_hInstance,
									NULL);
	}

	SendMessage(
		m_hwndMessageTextBox,
		EM_LIMITTEXT,
		0x00FFFFFF,
		0);

	//
    // You can set text box color and style
    //
}



/*++

Routine Name:
	
	SetMessageTextBoxText

Routine Description:
	
	Puts the decoded message text in the edit or
	rich edit control

Returns:

	None

--*/
void
CYArchDecode::SetMessageTextBoxText(
	LPTSTR inText)
{
	if (m_RichEdit)
	{
		gMsgText	= inText;
		gMsgTextPos	= 0;

		EDITSTREAM es = {0};
		es.pfnCallback = EditStreamCallback;

		SendMessage(
				m_hwndMessageTextBox,
				EM_STREAMIN,
				(WPARAM)SF_RTF,
				(LPARAM)&es);
	}
	else
	{
		SetWindowText(m_hwndMessageTextBox, inText);
	}
}


/*++

Routine Name:
	
	ReplaceChar

Routine Description:
	
	Replace all instance of a given character with another string
	in a given STL string

Returns:

	None

--*/
void
CYArchDecode::ReplaceChar(
	tstring &ioText,
	TCHAR	inChar,
	LPTSTR	inReplacement)
{
	int index = 0;
	while (true)
	{
		index = ioText.find_first_of(inChar, index);
		if (index == -1)
		{
			break;
		}

		ioText.replace(index, 1, inReplacement);
		index += _tcslen(inReplacement);
	}
}



/*++

Routine Name:
	
	RemoveSpecialRTFChar

Routine Description:
	
	Remove special RTF characters from a given string
	and replace them with equivalent RTF representation

Returns:

	None

--*/
void
CYArchDecode::RemoveSpecialRTFChar(
	tstring &ioText)
{	
	ReplaceChar(ioText, _T('\\'), _T("\\\\")); 
	ReplaceChar(ioText, _T('{'), _T("\\{"));
	ReplaceChar(ioText, _T('}'), _T("\\}"));

	//
	// Another way to convert these characters is to replace
	// them like \ to \'5C etc.
	//
}



/*++

Routine Name:
	
	InitMessageTextString

Routine Description:
	
	Inititalize a string to be used as a header for decoded
	archived message

Returns:

	None

--*/
void
CYArchDecode::InitMessageTextString(
	tstring &ioEditBoxTextString,
	tstring &inFileName)
{
	if (m_RichEdit)
	{
		tstring fileName = inFileName;
		RemoveSpecialRTFChar(fileName);

		ioEditBoxTextString = _T("{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1033{\\fonttbl{\\f0\\fnil Courier New;}{\\f1\\fnil\\fcharset0 Courier New;}}{\\colortbl\\red0\\green64\\blue0;\\red0\\green0\\blue128;\\red0\\green128\\blue0;\\red0\\green0\\blue255;\\red0\\green0\\blue0;\\red215\\green0\\blue0;}");
		ioEditBoxTextString += _T(	"\\viewkind4\\uc1\\pard\\qc\\f0"
									"\\cf2\\fs18 | \\b IntellectualHeaven (R) Yahoo Message Archive Decoder\\b0  |\\par "
									"\\cf2\\fs18 |   Copyright (c) \\cf3\\ul http://www.intellectualheaven.com\\ul0\\cf2     |\\par "
									"\\cf2\\fs18 |   Developed by: Pankaj Garg. All Rights Reserved.    |\\par "
									"\\cf2\\fs18 --------------------------------------------------------\\par "
									"\\viewkind4\\uc1\\pard "
									"\\cf5\\fs16\\b Decoded File: \\b0 ") + 
									fileName + 
									_T("\\par\\par ");
	}
	else
	{
		ioEditBoxTextString = _T(	" | YahDecode (Yahoo Message Archive Decoder)       |\r\n"
									" | Copyright (c) http://www.intellectualheaven.com |\r\n"
									" | Developed by: Pankaj Garg. All Rights Reserved. |\r\n"
									" ---------------------------------------------------\r\n"
									" Decoded File: ") + inFileName + _T("\r\n\r\n");
	}
}


/*++

Routine Name:
	
	FinalizeMessageTextString

Routine Description:
	
	Finalizes a decoded	archived message string and adds
	a footer to it.

Returns:

	None

--*/
void
CYArchDecode::FinalizeMessageTextString(
	tstring &ioEditBoxTextString)
{
	if (m_RichEdit)
	{
		ioEditBoxTextString += _T("}");
	}
}



/*++

Routine Name:
	
	FormatDecodedString

Routine Description:
	
	Format the decoded archived message for edit control
	or rich-edit control

Returns:

	None

--*/
void
CYArchDecode::FormatDecodedString(
	tstring &decodedString)
{
	unsigned int index = 0;

	//
	// Convert special bold/italics/underline tags to RTF
	// style tags
	//
	while (true)
	{
		index = decodedString.find_first_of(_T('\x1b'), 0);
		if (index == -1)
		{
			break;
		}

		unsigned int index_end = index + 1;
		while(decodedString[index_end] != _T('m'))
		{
			index_end++;

			if (index_end >= decodedString.length())
			{
				// We don't want to replace characters
				// past the string
				index_end = decodedString.length() - 1;
				break;
			}
		}

		if (m_RichEdit)
		{
			tstring formatCode = decodedString.substr(index + 1, index_end - index);

			if (formatCode == _T("[1m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\b "));
			}
			else if (formatCode == _T("[x1m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\b0 "));
			}
			else if (formatCode == _T("[2m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\i "));
			}
			else if (formatCode == _T("[x2m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\i0 "));
			}
			else if (formatCode == _T("[4m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\ul "));
			}
			else if (formatCode == _T("[x4m"))
			{
				decodedString.replace(index, index_end - index + 1, _T("\\ul0 "));
			}
			else
			{
				decodedString.replace(index, index_end - index + 1, _T(""));
			}
		}
		else
		{
			decodedString.replace(index, index_end - index + 1, _T(""));
		}
	}

	index = 0;
	while (true)
	{
		index = decodedString.find_first_of(_T('<'), index);
		if (index == -1)
		{
			break;
		}

		unsigned int index_end = index + 1;
		while(decodedString[index_end] != _T('>'))
		{
			++index_end;

			if (index_end >= decodedString.length())
			{
				index_end = 0;
				break;
			}
		}

		if (index_end > 0)
		{
			decodedString.replace(index, index_end - index + 1, _T(""));
		}
		else
		{
			index = index + 1;
		}
	}

	//
	// convert newline characters appropriately
	//
	index = 0;
	while(index < decodedString.length())
	{
		if (m_RichEdit)
		{
			if (decodedString[index] == _T('\n'))
			{
				if (decodedString[index - 1] != _T('\r'))
				{
					decodedString.replace(index, 1, _T(" \\par "));
				}
				else
				{
					decodedString.replace(index - 1, 2, _T(" \\par "));
				}
			}
		}
		else
		{
			if (decodedString[index] == _T('\n') &&
				decodedString[index - 1] != _T('\r'))
			{
				decodedString.replace(index, 1, _T("\r\n"));
			}
		}

		++index;
	}

	if (m_RichEdit)
	{
		decodedString = decodedString + _T(" \\par ");
	}
	else
	{
		decodedString = decodedString + _T("       \r\n");
	}
}


/*++

Routine Name:
	
	FormatUserNameString

Routine Description:
	
	Format the user name to display in decoded archived
	message for edit control or rich-edit control

Returns:

	None

--*/
void
CYArchDecode::FormatUserNameString(
	tstring &userNameString,
	bool	dispSelfName)
{
	if (m_RichEdit)
	{
		if (dispSelfName)
		{
			userNameString = _T("\\b0\\ul0\\i0\\cf0\\fs18\\b ") + userNameString + _T("\\b0 \\cf2 \\fs18 ");
		}
		else
		{
			userNameString = _T("\\b0\\ul0\\i0\\cf1\\fs18\\b ") + userNameString + _T("\\b0 \\cf3 \\fs18 ");
		}
	}
}



/*++

Routine Name:
	
	PopulateTextBoxData

Routine Description:
	
	Add the decoded archive text to the text box

Returns:

	None

--*/
void
CYArchDecode::PopulateTextBoxData(
	ARCHIVE_FILE_DATA &archiveFileData)
{
	//
	// Clear the text box
	//
	SetMessageTextBoxText(_T(""));

	tstring editBoxTextString;

	//
	// Open the archive .dat file
	//
	HANDLE hArchiveFile = CreateFile(
								archiveFileData.archiveFile.c_str(),
								GENERIC_READ,
								FILE_SHARE_READ,
								NULL,
								OPEN_EXISTING,
								0, 0);

	if (hArchiveFile != INVALID_HANDLE_VALUE)
	{	
		InitMessageTextString(editBoxTextString, archiveFileData.archiveFile);

		DWORD	bytesRead = 0;
		bool	bContinue = true;

		//
		// We will now decode the archive for different kinds of
		// archives. The format for each kind of archive is almost
		// same but we display it changes. Right now only two types
		// of message types are known i.e conference and IM. I am not
		// sure what happens with alert because i am not able to
		// generate any archive for an alert :(
		//
		while (bContinue)
		{
			YAHOO_MSG_TYPE		msgType = YAHOO_MSG_UNKNOWN;
			YAHOO_ARCH_MSG_HDR	msgHdr;

			if (ReadFile(
					hArchiveFile,
					&msgHdr,
					sizeof(msgHdr),
					&bytesRead,
					NULL))
			{
				if (bytesRead != sizeof(msgHdr))
				{
					bContinue = false;
					continue;
				}
				if (msgHdr.code == 0x6 ||
					msgHdr.code == 0x20)
				{
					msgType = YAHOO_MSG_IM;
				}
				else if (	msgHdr.code == 0x19 ||
							msgHdr.code == 0x1A	||
							msgHdr.code == 0x1B	||
							msgHdr.code == 0x1D)
				{
					msgType = YAHOO_MSG_CONF;
				}

				YAHOO_ARCH_MSG_DATA msgData;

				if (!ReadYahooMsgData(hArchiveFile, msgData))
				{
					bContinue = false;
					continue;
				}

				if (msgData.dataLength > 0)
				{
					struct tm *pTime	= localtime((const time_t *)&msgHdr.time);

					//
					// Decode the encoded data
					//
					TCHAR *decodedData	= NULL;
					decodedData = new TCHAR[msgData.dataLength + 1];

					if (!decodedData)
					{
						bContinue = false;
						continue;
					}

					int userNameLength = archiveFileData.userName.length(); 
					int nameIndex = 0;
					for (unsigned int i = 0; i < msgData.dataLength; ++i)
					{
						decodedData[i] = msgData.data[i] ^ archiveFileData.userName[nameIndex++ % userNameLength];
					}
					decodedData[i] = 0;

					tstring decodedString = decodedData;

					delete decodedData;


					//
					// Get the user name to display
					//
					tstring dispName;
					bool	dispSelfName = true;
							
					if (msgHdr.received == 0)
					{
						dispName		= archiveFileData.profileName;
						dispSelfName	= true;
					}
					else
					{
						if (msgType == YAHOO_MSG_IM)
						{
							dispName		= archiveFileData.peerName;
							dispSelfName	= false;
						}
						else
						{
							TCHAR *peerName = new TCHAR[msgData.peerNameLength + 1];
							if (!peerName)
							{
								bContinue = false;
								continue;
							}

							memset(peerName, 0, msgData.peerNameLength + 1);

							for (unsigned int i = 0; i < msgData.peerNameLength; ++i)
							{
								peerName[i] = msgData.peerName[i];
							}

							dispName		= peerName;
							dispSelfName	= false;

							delete peerName;
						}
					}

					//
					// When user declines to a conf, we get a message
					// which cause blank name for dispName with logic
					// above. In such case Yahoo puts profile name in
					// there and so do we
					//
					if (dispName.length() == 0)
					{
						dispName		= archiveFileData.profileName;
						dispSelfName	= true;
					}

					

					TCHAR amPM = _T('A');
					if (pTime->tm_hour >= 12)
					{
						amPM = _T('P');
					}

					TCHAR messagePrefix[MAX_PATH];

					if (m_ShowTimestamp)
					{
						_stprintf(	messagePrefix,
									_T("%s (%02d:%02d:%02d %cM): "),
									dispName.c_str(),
									pTime->tm_hour, pTime->tm_min, pTime->tm_sec,
									amPM);
					}
					else
					{
						_stprintf(	messagePrefix,
									_T("%s: "),
									dispName.c_str());
					}

					tstring userNameString;
					userNameString = messagePrefix;

					RemoveSpecialRTFChar(userNameString);
					RemoveSpecialRTFChar(decodedString);

					FormatUserNameString(userNameString, dispSelfName);
					FormatDecodedString(decodedString);
					
					editBoxTextString = editBoxTextString + userNameString + decodedString;
				}
			}
			else
			{
				bContinue = false;
				continue;
			}
		}

		FinalizeMessageTextString(editBoxTextString);

		SetMessageTextBoxText((LPTSTR)editBoxTextString.c_str());

		CloseHandle(hArchiveFile);
	}
	else
	{
		// show error
		int i = GetLastError();
	}
}



/*++

Routine Name:
	
	ReadYahooMsgData

Routine Description:
	
	Reads the data part from a yahoo archived message

Returns:

	true	- if read was successful
	false	- failure due to any reason

--*/
bool
CYArchDecode::ReadYahooMsgData(
	HANDLE					hFile,
	YAHOO_ARCH_MSG_DATA		&archMsgData)
{
	memset(&archMsgData, 0, sizeof(YAHOO_ARCH_MSG_DATA));

	if (!ReadYahooLenDataPair(
				hFile,
				archMsgData.dataLength,
				archMsgData.data))
	{
		goto funcExit;
	}

	if (!ReadYahooLenDataPair(
				hFile,
				archMsgData.peerNameLength,
				archMsgData.peerName))
	{	
		goto funcExit;
	}

	return true;

funcExit:

	if (archMsgData.data)
	{
		delete archMsgData.data;
		archMsgData.data = NULL;
	}

	if (archMsgData.peerName)
	{
		delete archMsgData.peerName;
		archMsgData.peerName = NULL;
	}

	return false;
}


/*++

Routine Name:
	
	ReadYahooLenDataPair

Routine Description:
	
	Reads the length-data pair for a yahoo archived
	message

Returns:

	true	- if read was successful
	false	- failure due to any reason

--*/
bool
CYArchDecode::ReadYahooLenDataPair(
	HANDLE	hFile,
	UINT32	&length,
	BYTE*	&data)
{
	DWORD bytesRead;

	if (!ReadFile(
				hFile,
				&length,
				sizeof(length),
				&bytesRead,
				NULL))
	{
		goto funcExit;
	}

	if (bytesRead != sizeof(length))
	{
		goto funcExit;
	}

	if (length > 0)
	{
		data = new BYTE[length];

		if (!data)
		{
			goto funcExit;
		}

		if (!ReadFile(
					hFile,
					data,
					length,
					&bytesRead,
					NULL))
		{
			goto funcExit;
		}

		if (bytesRead != length)
		{
			goto funcExit;
		}
	}

	return true;

funcExit:

	if (data)
	{
		delete data;
		data = NULL;
	}

	return false;
}




/*++

Routine Name:
	
	SaveAppInformation

Routine Description:
	
	Save the applications information in registry

Returns:

	<XXX>

--*/
int
CYArchDecode::SaveAppInformation()
{
	DWORD dwStyle = GetWindowLong(m_hwnd, GWL_STYLE);
	
	if (dwStyle)
	{
		if ((dwStyle & WS_MAXIMIZE) == 0)
		{
			RECT rectMain;
			int nRetVal = 0;
		    
			if (GetWindowRect(  m_hwnd,
								&rectMain))
			{
				HKEY hKey;

				nRetVal = RegCreateKey(	HKEY_CURRENT_USER,
										APP_INFO_REG_KEY,
										&hKey);

				if (nRetVal != ERROR_SUCCESS)
				{
					return -1;
				}

				nRetVal = RegSetValueEx(	hKey,
											_T("X1"),
											0,
											REG_DWORD,
											(const unsigned char *)&rectMain.left,
											sizeof(rectMain.left));

				if (nRetVal != ERROR_SUCCESS)
				{
					RegCloseKey(hKey);
					return -1;
				}


				nRetVal = RegSetValueEx(	hKey,
											_T("Y1"),
											0,
											REG_DWORD,
											(const unsigned char *)&rectMain.top,
											sizeof(rectMain.top));

				if (nRetVal != ERROR_SUCCESS)
				{
					RegCloseKey(hKey);
					return -1;
				}


				nRetVal = RegSetValueEx(	hKey,
											_T("X2"),
											0,
											REG_DWORD,
											(const unsigned char *)&rectMain.right,
											sizeof(rectMain.right));

				if (nRetVal != ERROR_SUCCESS)
				{
					RegCloseKey(hKey);
					return -1;
				}

				nRetVal = RegSetValueEx(	hKey,
											_T("Y2"),
											0,
											REG_DWORD,
											(const unsigned char *)&rectMain.bottom,
											sizeof(rectMain.bottom));

				if (nRetVal != ERROR_SUCCESS)
				{
					RegCloseKey(hKey);
					return -1;
				}

				RegCloseKey(hKey);
			}
		}
	}

    return 0;
}



/*++

Routine Name:
	
	LoadAppInformation

Routine Description:
	
	Read application information from the registry

Returns:

	<XXX>

--*/
int
CYArchDecode::LoadAppInformation(RECT *pRect)
{
    int nRetVal = 0;
    
    HKEY hKey;
    unsigned long nDataSize = 0;

	nRetVal = RegOpenKey(	HKEY_CURRENT_USER,
							APP_INFO_REG_KEY,
							&hKey);

	if (nRetVal != ERROR_SUCCESS)
	{
		return -1;
	}

    nDataSize = sizeof(pRect->left);
	nRetVal = RegQueryValueEx(	hKey,
								_T("X1"),
								0,
								NULL,
								(unsigned char *)&pRect->left,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}


    nDataSize = sizeof(pRect->top);
    nRetVal = RegQueryValueEx(	hKey,
								_T("Y1"),
								0,
								NULL,
								(unsigned char *)&pRect->top,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}


    nDataSize = sizeof(pRect->right);
    nRetVal = RegQueryValueEx(	hKey,
								_T("X2"),
								0,
								NULL,
								(unsigned char *)&pRect->right,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}

    nDataSize = sizeof(pRect->bottom);
    nRetVal = RegQueryValueEx(	hKey,
								_T("Y2"),
								0,
								NULL,
								(unsigned char *)&pRect->bottom,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}
    RegCloseKey(hKey);

    return 0;
}




/*++

Routine Name:
	
	UserInfoDlgProc

Routine Description:
	
	Dialog box to show Yahoo User information

Returns:

	TRUE - message handled by us
	FALSE - Not handled

--*/
INT_PTR
CALLBACK
UserInfoDlgProc(
	HWND hDlg,
	UINT msg,
	WPARAM wParam,
	LPARAM lParam)
{
	static ARCHIVE_FILE_DATA *archiveFileData = NULL;
	int result = 0;

	switch(msg)
	{
		case WM_INITDIALOG:
		{
			//
			// Insert *specific* dialog initialization code here.
			// lParam can contain information required for initialization.
			// If dialog box is created using DialogBoxParam
			//
			HICON hIcon = LoadIcon(ghInstance, MAKEINTRESOURCE(IDI_APP_ICON));
		    SendMessage (hDlg, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)hIcon);
		    SendMessage (hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hIcon);
			CenterDialog(hDlg);

			archiveFileData = (ARCHIVE_FILE_DATA *)lParam;

			SetWindowText(
					GetDlgItem(hDlg, IDC_EDIT_YAHOO_ID),
					archiveFileData->userName.c_str());

			SetWindowText(
					GetDlgItem(hDlg, IDC_EDIT_FRIEND_ID),
					archiveFileData->peerName.c_str());

			break;
		}
		case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDOK:
				{
					//
					// Set result here and terminate the dialog
					//

					TCHAR dataBuffer[64] = {0};

					GetWindowText(
							GetDlgItem(hDlg, IDC_EDIT_YAHOO_ID),
							dataBuffer,
							sizeof(dataBuffer) / sizeof(TCHAR));

					archiveFileData->userName = dataBuffer;

					GetWindowText(
							GetDlgItem(hDlg, IDC_EDIT_FRIEND_ID),
							dataBuffer,
							sizeof(dataBuffer) / sizeof(TCHAR));

					archiveFileData->peerName = dataBuffer;

					EndDialog(hDlg, result);
                    return TRUE;
				}
				case IDCANCEL:
				{
					EndDialog(hDlg, -1);
                    return TRUE;
				}
			}

			break;
		}
	}

	//
	// Not handled by us
	//
	return FALSE;
}


/*++

Routine Name:
	
	AboutDlgProc

Routine Description:
	
	Skeleton dialog box to show About box

Returns:

	<XXX>

--*/
INT_PTR
CALLBACK
AboutDlgProc(
	HWND hDlg,
	UINT msg,
	WPARAM wParam,
	LPARAM lParam)
{
	int		result = 0;

	switch(msg)
	{
		case WM_INITDIALOG:
		{
			//
			// Insert *specific* dialog initialization code here.
			// lParam can contain information required for initialization.
			// If dialog box is created using DialogBoxParam
			//
			HICON hIcon = LoadIcon(ghInstance, MAKEINTRESOURCE(IDI_APP_ICON));
		    SendMessage (hDlg, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)hIcon);
		    SendMessage (hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hIcon);

			CenterDialog(hDlg);
			break;
		}
		case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDOK:
				{
					//
					// Set result here and terminate the dialog
					//
					EndDialog(hDlg, result);
                    return TRUE;
				}
				case IDCANCEL:
				{
					EndDialog(hDlg, -1);
                    return TRUE;
				}
			}

			break;
		}
	}

	//
	// Not handled by us
	//
	return FALSE;
}


/*++

Routine Name:
	
	CenterDialog

Routine Description:
	
	Moves the dialog box to the center of its
	parent. If parent is NULL then dialog box
	is moved to the center of the desktop

Returns:

	None

--*/
void
CenterDialog(
	HWND hwndDlg)
{
	HWND hwndOwner;

	if ((hwndOwner = GetParent(hwndDlg)) == NULL) 
	{
		hwndOwner = GetDesktopWindow(); 
	}

	RECT rcOwner;
	RECT rcDlg;
	RECT rc;

	GetWindowRect(hwndOwner, &rcOwner); 
	GetWindowRect(hwndDlg, &rcDlg); 
	CopyRect(&rc, &rcOwner); 

	//
	// Offset the owner and dialog box rectangles so that 
	// right and bottom values represent the width and 
	// height, and then offset the owner again to discard 
	// space taken up by the dialog box. 
	//
	OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
	OffsetRect(&rc, -rc.left, -rc.top); 
	OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 

	// The new position is the sum of half the remaining 
	// space and the owner's original position.
	SetWindowPos(
			hwndDlg, 
			HWND_TOP, 
			rcOwner.left + (rc.right / 2), 
			rcOwner.top + (rc.bottom / 2), 
			0, 0,          // ignores size arguments 
			SWP_NOSIZE);

	return;
}


DWORD
CALLBACK
EditStreamCallback(
	DWORD_PTR	dwCookie,
	LPBYTE		pbBuff,
	LONG		cb,
	LONG		*pcb)
{
	LONG remainingBytes = (gMsgText.length() * sizeof(TCHAR)) - gMsgTextPos;
	BYTE *inputData = (BYTE *)gMsgText.c_str();

	if (remainingBytes > 0)
	{
		if (remainingBytes > cb)
		{
			*pcb = cb;
		}
		else
		{
			*pcb = remainingBytes;
		}

		memcpy(
			pbBuff,
			&inputData[gMsgTextPos],
			*pcb);

		gMsgTextPos += *pcb;
	}
	else
	{
		*pcb = 0;
	}

	return 0;
}


//////////////////////////////
// UNUSED FUNCTIONS BELOW
//////////////////////////////


/*++

Routine Name:
	
	UpdateMenus

Routine Description:
	
	Enable/Disable/Check/Uncheck menus based on the application
	specific logic

Returns:

	None

--*/
void
CYArchDecode::UpdateMenus()
{
	//
    // Here you should update state of all the menu items
    //
    HMENU hmenu = GetMenu(m_hwnd);

	EnableMenuItem(
			(struct HMENU__ *)hmenu,
			ID_FILE_EXIT,
			MF_ENABLED);
}



/*++

Routine Name:
	
	UpdateStatusBarData

Routine Description:
	
	Populate/Update Status bar data

Returns:

	None

--*/
void
CYArchDecode::UpdateStatusBarData(tstring &szText)
{
    SendMessage(
		m_hwndStatus,
		SB_SETTEXT,
		(WPARAM)1,
		(LPARAM)szText.c_str());

	SendMessage(
		m_hwndStatus,
		SB_SETTEXT,
		(WPARAM)3,
		(LPARAM)szText.c_str());
}

